{% extends '!footer.html' %}

{% block extrafooter %}
<p>This material is based upon work supported by the National Science Foundation under
Grant No. <a href="https://md.ekstrandom.net/research/career">IIS 17-51278</a>. Any
opinions, findings, and conclusions or recommendations expressed in this material
are those of the author(s) and do not necessarily reflect the views of the
National Science Foundation.  This page has not been approved by
Boise State University and does not reflect official university positions.</p>
<script data-goatcounter="https://seedbank.goatcounter.com/count"
        async src="//gc.zgo.at/count.js"></script>
{% endblock %}
